unit Img_Draw;

interface

uses WIndows,kol,Share,Debug;

type
  TTypeGraphic = (tgRectangle, tgEllipse, tgLine, tgPoint, tgBitmap, tgText);
  
type
  THIImg = class(TDebug)
   public
     GFont:PGraphicTool;
     sFont:TFontRec;
     hMyFont:HFONT;
     LOGPIXELSX_, LOGPIXELSY_ : integer;
     bmp,src:PBitmap;
     x1,y1,x2,y2:smallint;

    _prop_Text:string;
    _data_Text:THI_Event;
    _prop_X:integer;
    _prop_Y:integer;
    _data_X:THI_Event;
    _data_Y:THI_Event;

    _data_Bitmap:THI_Event;
    _data_SourceBitmap:THI_Event;
    _prop_Size:integer;
    _data_Size:THI_Event;

    _prop_Style:TBrushStyle;

    _prop_Color:TColor;
    _data_Color:THI_Event;
    _prop_BgColor:integer;
    _data_BgColor:THI_Event;

    _prop_Transparent:boolean;
    _prop_TransparentColor:TColor;


    _data_Point1:THI_Event;
    _data_Point2:THI_Event;

    _prop_Point2AsOffset:boolean;
    _prop_DrawSource: procedure(var _Data:TData; Graph:TTypeGraphic) of object;
    _event_onDraw:THI_Event;

    destructor Destroy; override;
    procedure SetNewFont(Value:TFontRec);
    procedure TFontToGFont(Font:TFontRec; grFont:PGraphicTool);

    property _prop_Font:TFontRec write SetNewFont;
    procedure ReadXY(var _Data:TData; Graph:TTypeGraphic);
    procedure Paint(DC:HDC; _Data:TData; Graph:TTypeGraphic; Context:boolean);

    procedure _work_doFont(var Data:TData; Index:word);
    procedure Bitmap(var _Data:TData; Graph:TTypeGraphic);
    procedure Handle(var _Data:TData; Graph:TTypeGraphic);
    procedure Context(var _Data:TData; Graph:TTypeGraphic);
  end;

implementation

destructor THIImg.Destroy;
begin
   if Assigned(GFont) then GFont.free;
   DeleteObject(hMyFont);
   inherited;
end;

procedure THIImg._work_doFont;
begin
   if _IsFont(Data) then SetNewFont(pfontrec(Data.idata)^);
end;

procedure THIImg.SetNewFont;
begin
   sFont := Value;
   if Assigned(GFont) then GFont.free;
   GFont := NewFont;
   TFontToGFont(sFont, GFont);
   hMyFont := GFont.Handle;
end;

procedure THIImg.TFontToGFont;
begin
   GrFont.Color:= Font.Color;
   Share.SetFont(GrFont,Font.Style);
   GrFont.FontName:= Font.Name;
   GrFont.FontHeight:= _hi_SizeFnt(Font.Size);
   GrFont.FontCharset:= Font.CharSet;
end;

procedure THIImg.ReadXY;
var   p:cardinal;
begin
   if (Graph <> tgPoint) and (Graph <> tgBitmap) and (Graph <> tgText) then begin
      p := ReadInteger(_Data,_data_Point1,0);
      y1 := p shr 16;
      x1 := p and $FFFF;
      p := ReadInteger(_Data,_data_Point2,0);
      y2 := p shr 16;
      x2 := p and $FFFF;
      if _prop_Point2AsOffset then
       begin
          inc(x2,x1);
          inc(y2,y1);
       end;
   end else begin
      x1 := ReadInteger(_Data,_data_X,_prop_X);
      y1 := ReadInteger(_Data,_data_Y,_prop_Y);
   end;
end;

procedure THIImg.Bitmap;
var   dt:TData;
      DC:HDC;
begin
   dt := _Data;
   bmp := ReadBitmap(_Data,_data_Bitmap,nil);
   ReadXY(_Data, Graph);
   if (bmp <> nil) and (not bmp.Empty) then begin
      DC := Bmp.Canvas.Handle;
      Paint(DC, _Data, Graph, false);
   end;
   _hi_CreateEvent(_Data,@_event_onDraw,dt);
end;

procedure THIImg.Handle;
var   dc:HDC;
      wnd:HWND;
      dt:TData;
begin
   dt := _Data;
   Wnd := ReadInteger(_Data,_data_Bitmap,0);
   dc := GetDC(wnd);
   ReadXY(_Data, Graph);
   Paint(DC, _Data, Graph, false);
   ReleaseDC(Wnd,Dc);
   _hi_CreateEvent(_Data,@_event_onDraw,dt);
end;

procedure THIImg.Context;
var   dc,dc2:HDC;
      dt:TData;
begin
   dt := _Data;
   DC := ReadInteger(_Data,_data_Bitmap,0);
   DC2 := GetDC(0);
   LOGPIXELSX_ := GetDeviceCaps(DC2,LOGPIXELSX);
   LOGPIXELSY_ := GetDeviceCaps(DC2,LOGPIXELSY);
   ReadXY(_Data, Graph);
   x1 := x1 * 2540 div LOGPIXELSX_;
   y1 := not y1 * 2540 div LOGPIXELSY_;
   x2 := x2 * 2540 div LOGPIXELSX_;
   y2 := not y2 * 2540 div LOGPIXELSY_;
   Paint(DC, _Data, Graph, true);
   ReleaseDC(0,DC2);
   _hi_CreateEvent(_Data,@_event_onDraw,dt);
end;

procedure THIImg.Paint;
var   br:HBRUSH;
      pen:HPEN;
      c:TColor;
      i, j, wh, hh: integer;
      hdcMem:HDC;
      hdcBmp:HBITMAP;
      hOldFont:HFONT;
      s:string;
begin
   if (Graph = tgEllipse) or (Graph = tgRectangle) then begin
      pen := CreatePen(0,1,ReadInteger(_Data,_data_Color,_prop_Color));
      if _prop_Style = bsSolid then begin
         c := ReadInteger(_Data,_data_BgColor,_prop_BgColor);
         if integer(c) < 0 then c := GetSysColor(c and $FFFFFF);
         br := CreateSolidBrush(c);
      end else br := GetStockObject(NULL_BRUSH);
      SelectObject(DC,br);
      SelectObject(DC,Pen);
      if Graph = tgRectangle then
         Rectangle(DC,x1,y1,x2,y2)
      else
         Ellipse(DC,x1,y1,x2,y2);
      DeleteObject(br);
      DeleteObject(Pen);
   end else if (Graph = tgLine) then begin
      c := ReadInteger(_Data,_data_Color,_prop_Color);
      pen := CreatePen(0,ReadInteger(_Data,_data_Size,_prop_Size),c);
      SelectObject(DC,Pen);
      MoveToEx(DC,x1,y1,nil);
      LineTo(DC,x2,y2);
      DeleteObject(Pen);
   end else if (Graph = tgPoint) then begin
      SetPixel(DC,x1,y1,ReadInteger(_Data,_data_Color,_prop_Color));
      exit;
   end else if (Graph = tgText) then begin
      s := ReadString(_Data,_data_Text,_prop_Text);
      SetBkMode(DC, TRANSPARENT);
      SetTextColor(DC, Color2RGB(GFont.Color));
      hOldFont := SelectObject(DC, hMyFont);
      TextOut(DC, x1, y1, PChar(s), length(s));
      SelectObject(DC, hOldFont);
   end else begin
      src := ReadBitmap(_Data,_data_SourceBitmap,nil);
      if not Context then begin
         if _prop_Transparent then
            src.DrawTransparent(DC,x1,y1,_prop_TransparentColor)
         else
            src.Draw(DC,x1,y1);
      end else begin
         wh := src.width * 2540 div LOGPIXELSX_;
         hh := src.height * 2540 div LOGPIXELSY_;
         hdcMem:= CreateCompatibleDC(0);
         hdcBmp:= CreateCompatibleBitmap(DC, wh, hh);
         SelectObject(hdcMem, hdcBmp);
         SetStretchBltMode(hdcMem, COLORONCOLOR);
         StretchBlt(hdcMem, 0 , 0, wh, hh, src.Canvas.Handle, 0, 0, src.width, src.height, SRCCOPY);
         for j := 0 to hh - 1 do
            for i := 0 to wh - 1 do
               SetPixel(DC, x1+i, y1-j, GetPixel(hdcMem, i, j));
         DeleteDC(hdcMem);
         DeleteObject(hdcBmp);
      end;
   end;

end;

end.